# Copyright 2015 by Austin Peters


# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name :        Simple Edge Railing
# Description : creates "railing" as aseries of simple edges, no faces.
# Author :      Austin Peters
# Credits:		Roughly based on the Weld.rb by Rick Wilson 
# Usage :       Select continuous edges, then run the script (Plugins>Edge Railing)
#				Railing will be created
# Date :        December 11, 2015
# Type :		Geometry creation tool
# History:
#				0.2 (11.Dec.2015) - first update. lots of functionality added
#				0.1 (10.Dec.2015) - first alpha version

require 'sketchup.rb'

def build_rail
#SETUP THE BASIC VARIABLES
	model=Sketchup.active_model #activate the sketchup model
	ents=model.active_entities #activate the entities in the model (make them accessible to ruby)
	group = ents.add_group #create a group to put stuff in
	sel=model.selection #make the selected items accessible
	verts=[]
	edges=[]
	newVerts=[]
	railVerts=[]
	newvert=nil
	startEdge=startVert=nil
	picketSpace = "Absolute"
	picketSpacing = 72
	
#DELETE NON-EDGES, GET THE VERTICES
	sel.each {|item| edges.push(item) if item.typename=="Edge"}
	edges.each {|edge| verts.push(edge.vertices)}
	verts.flatten!
	return UI.messagebox("Please select at least 1 edge") if edges.length < 1

#Code below courtesy of Weld.rb
#FIND AN END VERTEX
	vertsShort=[]
	vertsLong=[]
	verts.each do |v|
		if vertsLong.include?(v)
			vertsShort.push(v)
		else
			vertsLong.push(v)
		end
	end
	if (startVert=(vertsLong-vertsShort).first)==nil
		startVert=vertsLong.first
		closed=true
		startEdge = startVert.edges.first
	else
		closed=false
		startEdge = (edges & startVert.edges).first
	end

#SORT VERTICES, LIMITING TO THOSE IN THE SELECTION SET
	if startVert==startEdge.start
		newVerts=[startVert]
		counter=0
		while newVerts.length < verts.length
			edges.each do |edge|
				if edge.end==newVerts.last
					newVerts.push(edge.start)
				elsif edge.start==newVerts.last
					newVerts.push(edge.end)
				end
			end
			counter+=1
			if counter > verts.length
				return nil if UI.messagebox("There seems to be a problem. Try again?", MB_YESNO)!=6
				newVerts.reverse!
				reversed=true
			end
		end
	else
		newVerts=[startVert]
		counter=0
		while newVerts.length < verts.length
			edges.each do |edge|
				if edge.end==newVerts.last
					newVerts.push(edge.start)
				elsif edge.start==newVerts.last
					newVerts.push(edge.end)
				end
			end
			counter+=1
			if counter > verts.length
				return nil if UI.messagebox("There seems to be a problem. Try again?", MB_YESNO)!=6
				newVerts.reverse!
				reversed=true
			end
		end
	end
	newVerts.uniq!
	newVerts.reverse! if reversed
#end Weld.rb borrowed code

# GET SOME PRELIMINARY INPUT
	#define an integer check, to keep things honest
	def checkInt(foo)
		if foo.is_a? Integer
			return true
		else 
			UI.messagebox("You must enter an integer")
			return false
		end
	end
	#some code to help keep the loop open or closed, depending on what has already been drawn
	if closed == false
		defClosed = "No"
	else
		defClosed = "Yes"
	end
	#get some input
	def getInput(defClosed)
		prompts = ["Number of Horizontal Rails:", "Total Height of Rails, in Inches:", "Picket Spacing, feet:", "Picket Spacing Type:", "Close Loop (or keep it closed):", "Group Construction Lines:", "Delete Construction Lines:"]
		defaults = [3,36, 6, "Absolute", defClosed, "No", "Yes"]
		lists = ["","", "","Absolute|Interpreted", "Yes|No", "Yes|No", "Yes|No"]
		begin
			inputs = UI.inputbox(prompts, defaults, lists, "Simple Railing Setup")
		rescue
			UI.messagebox("You must enter an integer")
			getInput()
		end
		(inputs) ? () : return
		checkInt(inputs[0]) ? () : getInput()
		checkInt(inputs[1]) ? () : getInput()
		return inputs #inputs are returned as an array
	end
	#assign the input results array to individual variables for use later
	inputs = getInput(defClosed)
	(inputs) ? () : return
	numRails = inputs[0]
	railHeight = inputs[1]
	picketSpacing = inputs[2] * 12
	picketSpace = inputs[3]
	if inputs[4] == "Yes"
		closed = true
	else
		closed = false
	end
	groupLines = inputs[5]
	deleteLines = inputs[6]

#CONVERT VERTICES TO POINT3Ds
	newVerts.collect!{|x| x.position}
	if closed == true
		newVerts.push(newVerts[0])
		baseline = ents.add_line(newVerts[0],newVerts[-2]) #add the closed 'base' line, for completeness
		sel.add baseline
	end

#CONVERT THE VERTICES TO RAILS
	model.start_operation "Building Railings"
	railLength = newVerts.length - 2 #this seems to be the right number of times
	railNum = 1 #for some reason an external counter works better than using a built in function
	(1..numRails).each do |k|
		newOffset = (railHeight / numRails * railNum) #define the offset for the next row of rails
		railOffset = Geom::Vector3d.new 0, 0, newOffset #turn that offset into a vector
		newVerts.each_with_index do |vert, j|
			railVerts[j] = (vert + railOffset) #create a new set of vertices for the new railing
		end
		(0..railLength).each do |i| #draw the new railing
			group.entities.add_line(railVerts[i],railVerts[i+1]) #add lower rail
		end
		railNum += 1
	end
	#ADD VERTICAL PICKETS
	#DETERMINE PICKET SPACING, IF INTERPRETED
	
	
	#FINALLY MAKE SOME PICKETS!
	(0..railLength).each do |m|
		segVerts = newVerts[m+1] - newVerts[m]
		segVector = Geom::Vector3d.new segVerts
		segLength = segVector.length
		if picketSpace == "Interpreted"
			numPickets = segLength / picketSpacing
			puts(numPickets)
			numPickets = (numPickets + 1)
			numPickets = numPickets.to_i
			picketSpacing = segLength / numPickets
		end
		picketLocVector = segVector.normalize
		picketLocVector.length = picketSpacing
		numPickets = (segLength / picketSpacing).to_i
		picketVert = newVerts[m]
		(0..(numPickets)).each do |n|
			picketTopVert = picketVert + [0,0,railHeight]
			group.entities.add_line(picketVert, picketTopVert)
			picketVert = picketVert + [picketLocVector[0], picketLocVector[1], picketLocVector[2]]
		end
	end
	#do the last picket
	picketTopVert = newVerts[-1] + [0,0,railHeight]
	group.entities.add_line(newVerts[-1], picketTopVert)
	
	#delete construction lines, if asked to
	if deleteLines == "Yes"
		group.entities.erase_entities(sel)
	end
	if groupLines == "Yes"
		constructionGroup = group.entities.add_group(sel)
		group.entities.add_group(constructionGroup, group)
	end
	
	model.commit_operation
	#sel.clear
end
#-----------------------------------------------------------------------------
if( not file_loaded?("edge_railing.rb") )
    add_separator_to_menu("Plugins")
	# BEGIN CHANGES BY organizerEdit.rb
	if $submenu!=nil
		$submenu.add_item("Simple Railing") { build_rail }
	else
		UI.menu("Plugins").add_item("Simple Railing") { build_rail }
	end
	# END CHANGES BY organizerEdit.rb
end
#-----------------------------------------------------------------------------
file_loaded("edge_railing.rb")
